@@ -14,6 +14,7 @@
   limitations under the License.
 */
 
+use clap::Parser;
 use k::nalgebra as na;
 use k::prelude::*;
 use kiss3d::event::{Action, Key, Modifiers, WindowEvent};
@@ -22,7 +23,6 @@ use serde::Deserialize;
 use std::fmt;
 use std::path::PathBuf;
 use std::sync::Arc;
-use structopt::StructOpt;
 use tracing::*;
 
 #[cfg(not(target_arch = "wasm32"))]
@@ -144,7 +144,7 @@ impl UrdfViewerApp {
     ) -> Result<Self, Error> {
         let input_path = PathBuf::from(&urdf_robot.path);
         let robot: k::Chain<f32> = urdf_robot.get().into();
-        println!("{}", robot);
+        println!("{robot}");
         let (mut viewer, mut window) = Viewer::with_background_color("urdf-viz", background_color);
         if disable_texture {
             viewer.disable_texture();
@@ -171,17 +171,17 @@ impl UrdfViewerApp {
             .iter()
             .filter_map(|name| robot.find(name).map(k::SerialChain::from_end))
             .collect::<Vec<_>>();
-        println!("end_link_names = {:?}", end_link_names);
+        println!("end_link_names = {end_link_names:?}");
         let names = robot
             .iter_joints()
             .map(|j| j.name.clone())
             .collect::<Vec<_>>();
         let num_arms = end_link_names.len();
         let num_joints = names.len();
-        println!("DoF={}", num_joints);
-        println!("joint names={:?}", names);
+        println!("DoF={num_joints}");
+        println!("joint names={names:?}");
         let mut handle = RobotStateHandle::default();
-        handle.current_joint_positions.write().unwrap().names = names.clone();
+        handle.current_joint_positions.lock().names = names.clone();
         handle.urdf_text = Some(urdf_robot.urdf_text.clone());
         Ok(UrdfViewerApp {
             input_path,
@@ -243,7 +243,7 @@ impl UrdfViewerApp {
         let joint_positions = self.robot.joint_positions();
         self.robot
             .set_joint_positions(&joint_positions)
-            .unwrap_or_else(|err| error!("failed to update robot joints {}", err));
+            .unwrap_or_else(|err| error!("failed to update robot joints {err}"));
         self.viewer.update(&self.robot);
         self.update_ik_target_marker();
     }
@@ -275,7 +275,7 @@ impl UrdfViewerApp {
         if self.names != names {
             let dof = names.len();
             self.names = names.clone();
-            let mut current_joint_positions = self.handle.current_joint_positions.write().unwrap();
+            let mut current_joint_positions = self.handle.current_joint_positions.lock();
             current_joint_positions.names = names;
             current_joint_positions.positions = vec![0.0; dof];
         }
@@ -303,7 +303,7 @@ impl UrdfViewerApp {
             if let Some(index) = self.names.iter().position(|n| n == name) {
                 angles[index] = *angle;
             } else {
-                warn!("{} not found, but continues", name);
+                warn!("{name} not found, but continues");
             }
         }
         self.robot.set_joint_positions(&angles)
@@ -507,19 +507,18 @@ impl AppState {
                         self.app.update_robot();
                     }
                     Err(err) => {
-                        error!("{}", err);
+                        error!("{err}");
                     }
                 }
             }
-            handle.current_joint_positions.write().unwrap().positions =
-                self.app.robot.joint_positions();
+            handle.current_joint_positions.lock().positions = self.app.robot.joint_positions();
 
             // Robot orientation for server
             if let Some(ro) = handle.take_target_robot_origin() {
                 self.app.set_robot_origin_from_request(&ro);
                 self.app.update_robot();
             }
-            let mut cur_ro = handle.current_robot_origin.write().unwrap();
+            let mut cur_ro = handle.current_robot_origin.lock();
             let o = self.app.robot.origin();
             for i in 0..3 {
                 cur_ro.position[i] = o.translation.vector[i];
@@ -575,7 +574,7 @@ impl window::State for AppState {
                 .to_owned();
             self.app.viewer.draw_text(
                 window,
-                &format!("IK target name [{}]", name),
+                &format!("IK target name [{name}]"),
                 FONT_SIZE_INFO,
                 &na::Point2::new(10f32, 100.0),
                 &na::Point3::new(0.5f32, 0.8, 0.2),
@@ -653,7 +652,7 @@ impl window::State for AppState {
                                 )
                                 .unwrap_or_else(|err| {
                                     self.app.robot.set_joint_positions_unchecked(&orig_angles);
-                                    error!("{}", err);
+                                    error!("{err}");
                                 });
                             self.app.update_robot();
                         }
@@ -697,8 +696,8 @@ impl window::State for AppState {
     }
 }
 
-/// Option for visualizing urdf
-#[derive(StructOpt, Debug, Deserialize)]
+/// URDF visualizer
+#[derive(Parser, Debug, Deserialize)]
 #[serde(rename_all = "kebab-case")]
 #[non_exhaustive]
 pub struct Opt {
@@ -706,73 +705,73 @@ pub struct Opt {
     #[serde(default, rename = "urdf")]
     pub input_urdf_or_xacro: String,
     /// end link names
-    #[structopt(short = "e", long = "end-link-name")]
+    #[clap(short, long = "end-link-name")]
     #[serde(default)]
     pub end_link_names: Vec<String>,
     /// Show collision element instead of visual
-    #[structopt(short = "c", long = "collision")]
+    #[clap(short = 'c', long = "collision")]
     #[serde(default)]
     pub is_collision: bool,
     /// Disable texture rendering
-    #[structopt(short = "d", long = "disable-texture")]
+    #[clap(short, long)]
     #[serde(default)]
     pub disable_texture: bool,
     /// Port number for web server interface
-    #[structopt(short = "p", long = "web-server-port", default_value = "7777")]
+    #[clap(short = 'p', long, default_value = "7777")]
     #[serde(default = "default_web_server_port")]
     pub web_server_port: u16,
 
-    #[structopt(long = "ignore-ik-position-x")]
+    #[clap(long)]
     #[serde(default)]
     pub ignore_ik_position_x: bool,
-    #[structopt(long = "ignore-ik-position-y")]
+    #[clap(long)]
     #[serde(default)]
     pub ignore_ik_position_y: bool,
-    #[structopt(long = "ignore-ik-position-z")]
+    #[clap(long)]
     #[serde(default)]
     pub ignore_ik_position_z: bool,
 
-    #[structopt(long = "ignore-ik-rotation-x")]
+    #[clap(long)]
     #[serde(default)]
     pub ignore_ik_rotation_x: bool,
-    #[structopt(long = "ignore-ik-rotation-y")]
+    #[clap(long)]
     #[serde(default)]
     pub ignore_ik_rotation_y: bool,
-    #[structopt(long = "ignore-ik-rotation-z")]
+    #[clap(long)]
     #[serde(default)]
     pub ignore_ik_rotation_z: bool,
 
-    #[structopt(long = "bg-color-r", default_value = "0.0")]
+    #[clap(long = "bg-color-r", default_value = "0.0")]
     #[serde(default)]
     pub back_ground_color_r: f32,
-    #[structopt(long = "bg-color-g", default_value = "0.0")]
+    #[clap(long = "bg-color-g", default_value = "0.0")]
     #[serde(default)]
     pub back_ground_color_g: f32,
-    #[structopt(long = "bg-color-b", default_value = "0.3")]
+    #[clap(long = "bg-color-b", default_value = "0.3")]
     #[serde(default = "default_back_ground_color_b")]
     pub back_ground_color_b: f32,
 
-    #[structopt(long = "tile-color1-r", default_value = "0.1")]
+    #[clap(long, default_value = "0.1")]
     #[serde(default = "default_tile_color1")]
     pub tile_color1_r: f32,
-    #[structopt(long = "tile-color1-g", default_value = "0.1")]
+    #[clap(long, default_value = "0.1")]
     #[serde(default = "default_tile_color1")]
     pub tile_color1_g: f32,
-    #[structopt(long = "tile-color1-b", default_value = "0.1")]
+    #[clap(long, default_value = "0.1")]
     #[serde(default = "default_tile_color1")]
     pub tile_color1_b: f32,
 
-    #[structopt(long = "tile-color2-r", default_value = "0.8")]
+    #[clap(long, default_value = "0.8")]
     #[serde(default = "default_tile_color2")]
     pub tile_color2_r: f32,
-    #[structopt(long = "tile-color2-g", default_value = "0.8")]
+    #[clap(long, default_value = "0.8")]
     #[serde(default = "default_tile_color2")]
     pub tile_color2_g: f32,
-    #[structopt(long = "tile-color2-b", default_value = "0.8")]
+    #[clap(long, default_value = "0.8")]
     #[serde(default = "default_tile_color2")]
     pub tile_color2_b: f32,
 
-    #[structopt(long = "ground-height")]
+    #[clap(long)]
     pub ground_height: Option<f32>,
 }
 
@@ -805,8 +804,20 @@ impl Opt {
     #[cfg(target_arch = "wasm32")]
     pub fn from_params() -> Result<Self, Error> {
         let href = crate::utils::window()?.location().href()?;
-        debug!("href={}", href);
+        debug!("href={href}");
         let url = url::Url::parse(&href).map_err(|e| e.to_string())?;
         Ok(serde_qs::from_str(url.query().unwrap_or_default()).map_err(|e| e.to_string())?)
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use clap::IntoApp;
+
+    use super::*;
+
+    #[test]
+    fn assert_app() {
+        Opt::into_app().debug_assert();
+    }
+}
